package br.com.gestor.controller;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.ApplicationPath;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Application;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;

import br.com.gestor.bean.Quadras;
import br.com.gestor.bean.TipoQuadra;
import br.com.gestor.dao.QuadrasDao;

@ApplicationPath("/minhaquadra")//TEM BARRA INICIAL
@Path("quadra")//NAO TEM BARRA INICIAL
public class QuadrasCtrl extends Application {

   private static final int SUCCESS_RECORD = 1;
   private static final int FAILURE_RECORD = 0;

    //http://localhost:8080/quadras/minhaquadra/quadra/console/123
    @GET
	@Path("console/{objeto}")
	public String getQuadras(@PathParam("objeto") int n){
		System.err.println(n);
		return String.valueOf(n);
	}

	//http://localhost:8080/quadras/minhaquadra/quadra/cadastrar/banco
	@PUT
	@Path("cadastrar/banco")
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)//APPLICATION_FORM_URLENCODED NAO ACEITA @GET
	public int cadastroQuadras(
			@FormParam("codQuadra") int codQuadra,
			@FormParam("tipo") String tipoString,
			@FormParam("descricao") String descricao,
			@FormParam("eDisponivel") int eDisponivel,
			@FormParam("eCoberta") int eCoberta,
			@Context HttpServletResponse servletResponse) throws IOException{

		TipoQuadra tipo = new TipoQuadra();
		tipo.setDescricao(tipoString);
		tipo.buscar();
		
		Quadras quadra = new Quadras(codQuadra, tipo.getCodTipo(), descricao, eDisponivel, eCoberta, null);

		int result = quadra.cadastrar();
		if(result>0)
			return SUCCESS_RECORD;
		return FAILURE_RECORD;
	}

	//http://localhost:8080/quadras/minhaquadra/quadra/cadastrar/tipo
	@PUT
	@Path("cadastrar/tipo")
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)
	public int cadastrosTiposQuadras(
			@FormParam("codTipo") int codTipo,
			@FormParam("descricao") String descricao,
			@Context HttpServletResponse servletResponse) throws IOException{

		TipoQuadra tipo = new TipoQuadra();
		tipo.setCodTipo(codTipo);
		tipo.setDescricao(descricao);

		int result = tipo.cadastrar();
		if(result>0)
			return SUCCESS_RECORD;
		return FAILURE_RECORD;
	}

	//http://localhost:8080/quadras/minhaquadra/quadra/excluir/banco
	@DELETE
	@Path("excluir/banco")
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)
	public int excluirQuadras(
			@FormParam("codQuadra") int codQuadra,
			@Context HttpServletResponse servletResponse) throws IOException{

		Quadras quadra = new Quadras();
		quadra.setCodQuadra(codQuadra);
		
		int result = quadra.deletar();
		if(result>0)
			return SUCCESS_RECORD;
		return FAILURE_RECORD;
	}

    //http://localhost:8080/quadras/minhaquadra/quadra/buscar/esporte
    @POST
	@Path("buscar/esporte/hora")
	@Produces(MediaType.TEXT_PLAIN)
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)//Pega com @FormParam e NAO @PathParam
	public String getListarQuadrasHora(
			@FormParam("esporte") String esporte,
			@FormParam("horaInicial") String horaInicialParam,
			@FormParam("horaFinal") String horaFinalParam,
			@Context HttpServletResponse servletResponse) throws IOException, ParseException
		{

		SimpleDateFormat horaFormat = new SimpleDateFormat("HH:mm");
		Date horaInicial = horaFormat.parse(horaInicialParam);
		Date horaFinal = horaFormat.parse(horaFinalParam);
    	
    	QuadrasDao dao = new QuadrasDao();
    	ArrayList<Quadras> lista = dao.pesquisarPorQuadrasDisponiveisViaHORA(esporte,horaInicial,horaFinal);
    	
    	String tokens = "";
    	for (Quadras quadras : lista) {
    		if(quadras.geteDisponivel()==0)
    			tokens = quadras.getCodQuadra()+";"+quadras.getDescricao()+";Disponível" + ";" + tokens;
    		else
    			tokens = quadras.getCodQuadra()+";"+quadras.getDescricao()+";Fechada" + ";" + tokens;//Esta fechada ou em manutenção, pode não ter reservas ou ter para manutencao.
		}
    	//remove o ultimo ponto e virgula
    	if(tokens.length() > 1)
    		tokens = tokens.substring(0,tokens.length()-1);
    	
    	return tokens;    	
	}
	
    //http://localhost:8080/quadras/minhaquadra/quadra/buscar/esporte
    @POST
	@Path("buscar/esporte/data")
	@Produces(MediaType.TEXT_PLAIN)
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)//Pega com @FormParam e NAO @PathParam
	public String getListarQuadrasData(
			@FormParam("esporte") String esporte,
			@FormParam("dataInicial") String dataInicialParam,
			@FormParam("dataFinal") String dataFinalParam,
			@FormParam("horaInicial") String horaInicialParam,
			@FormParam("horaFinal") String horaFinalParam,
			@Context HttpServletResponse servletResponse) throws IOException, ParseException
		{
    	
		SimpleDateFormat dataFormat = new SimpleDateFormat("dd/MM/yyyy");
		Date dataInicial = dataFormat.parse(dataInicialParam);
		Date dataFinal = dataFormat.parse(dataFinalParam);
		
		SimpleDateFormat horaFormat = new SimpleDateFormat("HH:mm");
		Date horaInicial = horaFormat.parse(horaInicialParam);
		Date horaFinal = horaFormat.parse(horaFinalParam);
    	

    	QuadrasDao dao = new QuadrasDao();
    	ArrayList<Quadras> lista = dao.pesquisarPorQuadrasDisponiveisViaData(esporte,dataInicial,dataFinal,horaInicial,horaFinal);
    	
    	String tokens = "";
    	for (Quadras quadras : lista) {
    		if(quadras.geteDisponivel()==0)
    			tokens = quadras.getCodQuadra()+";"+quadras.getDescricao()+";Disponível" + ";" + tokens;
    		else
    			tokens = quadras.getCodQuadra()+";"+quadras.getDescricao()+";Fechada" + ";" + tokens;//Esta fechada ou em manutenção, pode não ter reservas ou ter para manutencao.
		}
    	//remove o ultimo ponto e virgula
    	if(tokens.length() > 1)
    		tokens = tokens.substring(0,tokens.length()-1);
    	
    	return tokens;    	
	}    
}
