﻿package br.com.gestor.model;

import java.io.IOException;
import java.io.InputStream;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;

import org.apache.log4j.Logger;

/*
 * Classe para conexao com o banco de dados SQLSERVER
 * O contexto e estatico somente uma conexao a mesma aberta sera a fechada.
 *		ConnectionFactory.openConnection();
 *		ConnectionFactory.closeConnection();
 */

public class ConnectionFactory {

	public static Connection connection = null;
	private Properties prop = null; 
	static Logger log = Logger.getLogger(ConnectionFactory.class);
	private final String fileProperties = "database.properties";//ADICIONAR A PASTA resources AO BUILD PATH ou somente o arquivo.properties
                                                                //se o build do projeto for via Maven(nao existe buildpath) basta colocar na pasta main/resources que o maven cria.

	public void open() {

	        prop = getProperties();
	
	        //System.out.println("-------- JDBC Connection ------");
	        String recurso = prop.getProperty("database.recurso");
	        String driver = prop.getProperty("database.driver");
	        String url = prop.getProperty("database.url");
	        String dbname = prop.getProperty("database.dbname");
	        String usuario = prop.getProperty("database.usuario");
	        String senha = prop.getProperty("database.senha");
	
	        try {
	        	try {
					Class.forName(driver);
				} catch (ClassNotFoundException ex) {
		            log.error("Erro ao carregar o driver. ", new RuntimeException(ex.getMessage()));
				}
	            connection = DriverManager.getConnection( recurso + url + dbname, usuario, senha );
	        } catch (SQLException ex) {
	            log.error("Erro na conexão com o banco de dados: ", new RuntimeException(ex.getMessage()));
	        }
	}

	public Properties getProperties(){

			Properties proper = new Properties();
			InputStream input = null;
	       try {
	            input = getClass().getClassLoader().getResourceAsStream(fileProperties);//trata-se da raiz do pacote src/main...
	
	            if(input!=null)
	                proper.load(input);
	            else
	                System.out.println("Nao foi possivel encontrar o arquivo de properties\nVerifique se o mesmo foi adicionado ao Build Path do projeto!\n\n"+fileProperties+"\n\n");
	            	//java.util.Enumeration<?> e = proper.propertyNames(); -- Listar todas as propriedades
	       }
	       catch(java.io.FileNotFoundException e){ System.err.println("Erro Arquivo não encontrado. : "+e.getMessage()); }
	       catch (IOException ex) { log.error("Erro de abertura e fechamento de arquivo. : ", new RuntimeException(ex.getMessage()));}
	       finally{
	            if(input!=null){
	                try { input.close(); }
	                catch (IOException ex) { log.error("Erro ao fechar IO: ", new RuntimeException(ex.getMessage()));}
	            }
	        }
	
	        return proper;
	}

	//Outro método pq getClass().getClassLoader(), nao pode assumir um contexto estatico
	public static Connection abrirConexoes() {
            try {
				if (connection == null || connection.isClosed()){
				    ConnectionFactory c = new ConnectionFactory();
				    c.open();
				}
			} catch (SQLException ex) {
				log.error("Erro ao conectar ao banco de dados, verifique se o banco esta startado.", new RuntimeException(ex.getMessage()));
			}
            return connection;
	}

    public static void fecharConexoes(ResultSet rset, CallableStatement cstm, PreparedStatement pstm, Connection conn) {
        try{if(rset!= null){rset.close();}}catch(Exception ex){log.info("Erro ao fechar o ResultSet. Message="+ex.getMessage());}
        try{if(cstm!= null){cstm.close();}}catch(Exception ex){log.info("Erro ao fechar o CallableStatement. Message="+ex.getMessage());}
        try{if(pstm!= null){cstm.close();}}catch(Exception ex){log.info("Erro ao fechar o PreparedStatement. Message="+ex.getMessage());}
        try{if(conn!= null){conn.close();}}catch(Exception ex){log.info("Erro ao fechar o Connection. Message="+ex.getMessage());}
	}

}
